@@ -168,7 +168,7 @@ def classify_features(dfte, depVar, model_options={}, verbose=0):
     elif verbose==1 and len(preds) > max_cols_analyzed:
         print('   Total columns > %d, too numerous to list.' %max_cols_analyzed)
     features_dict = dict([('IDcols',IDcols),('cols_delete',cols_delete),('categorical_vars',categorical_vars), (
-                        'lats',lats),('lons',lons),('matched_pairs',matched_pairs), ('int_vars',int_vars),
+                        'lat_vars',lats),('lon_vars',lons),('matched_pairs',matched_pairs), ('int_vars',int_vars),
                         ('continuous_vars',continuous_vars),('discrete_string_vars',discrete_string_vars),
                         ('nlp_vars',nlp_vars), ('date_vars',date_vars)])
     return features_dict
@@ -488,6 +488,7 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
     feats_max_min = nested_dictionary()
     print_features = False
     nlps = []
+    bools = []
     ### if a variable has more than this many chars, it will be treated like a NLP variable
     nlp_char_limit = check_model_options(model_options, "nlp_char_limit", 30)
     ### if a variable has more than this limit, it will not be treated like a cat variable #
@@ -497,8 +498,8 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
     dates = var_df1['date_vars']
     cats = var_df1['categorical_vars']
     discrete_strings = var_df1['discrete_string_vars']
-    lats = var_df1['lats']
-    lons = var_df1['lons']
+    lats = var_df1['lat_vars']
+    lons = var_df1['lon_vars']
     ignore_variables = var_df1['cols_delete']
     all_ints = var_df1['int_vars']
     if isinstance(target, list):
@@ -517,13 +518,17 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
     ###### Now we do the creation of cat_vocab_dict though it is called feats_max_min here #####
     floats = []
     for key in preds:
-        if data_sample[key].dtype == 'object' or str(data_sample[key].dtype) == 'category':
+        if data_sample[key].dtype in ['object'] or str(data_sample[key].dtype) == 'category':
             feats_max_min[key]["dtype"] = "string"
+        elif data_sample[key].dtype in ['bool']:
+            feats_max_min[key]["dtype"] = "bool"
+            bools.append(key)
+            if key in cats:
+                cats.remove(key)
         elif str(data_sample[key].dtype).split("[")[0] in ['datetime64','datetime32','datetime16']:
             feats_max_min[key]["dtype"] = "string"
-        elif data_sample[key].dtype in [np.int16, np.int32,
-                                np.int64]:
-            feats_max_min[key]["dtype"] = "int32"
+        elif data_sample[key].dtype in [np.int16, np.int32, np.int64]:
+            feats_max_min[key]["dtype"] = data_sample[key].dtype
         else:
             floats.append(key)
             feats_max_min[key]["dtype"] = data_sample[key].dtype
@@ -543,14 +548,20 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
                     feats_max_min[key]["vocab"] = vocab
                     feats_max_min[key]['size_of_vocab'] = len(vocab)
                 else:
-                    ### For the rest of the numeric variables, you don't need vocab ###
-                    #feats_max_min[key]["vocab"] = data_sample[key].unique()
+                    ### For the rest of the numeric variables, you just need mean and variance ###
                     vocab = data_sample[key].unique()
-                    feats_max_min[key]["vocab"] = []
+                    feats_max_min[key]["vocab_min_var"] = [data_sample[key].mean(), data_sample[key].var()]
+                    feats_max_min[key]["vocab"] = vocab
                     feats_max_min[key]['size_of_vocab'] = len(vocab)
             feats_max_min[key]["max"] = max(data_sample[key].values)
             feats_max_min[key]["min"] = min(data_sample[key].values)
-        elif feats_max_min[key]['dtype'] == 'string':
+        elif feats_max_min[key]['dtype'] in ['bool']:
+            vocab = data_sample[key].unique().astype(str)
+            ### Don't change the next line even though it appears wrong. I have tested and it works!
+            vocab = ['missing' if type(x) == float else x for x in vocab]
+            feats_max_min[key]["vocab"] = vocab
+            feats_max_min[key]['size_of_vocab'] = len(vocab)
+        elif feats_max_min[key]['dtype'] in ['string']:
             if np.mean(data_sample[key].fillna("missing").map(len)) >= nlp_char_limit:
                 ### This is for NLP variables. You want to remove duplicates #####
                 if key in dates:
@@ -578,10 +589,12 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
                 feats_max_min[key]['size_of_vocab'] = len(vocab)
                 #feats_max_min[key]['size_of_vocab'] = len(feats_max_min[key]["vocab"])
         else:
-            ####  Now we select features if they are present in the data set ###
+            ####  Now we treat bool and other variable types ###
             #feats_max_min[key]["vocab"] = data_sample[key].unique()
             vocab = data_sample[key].unique()
-            feats_max_min[key]["vocab"] = []
+            #### just leave this as is - it works for other data types
+            vocab = ['missing' if type(x) == str  else x for x in vocab]
+            feats_max_min[key]["vocab"] = vocab
             feats_max_min[key]['size_of_vocab'] = len(vocab)
             #feats_max_min[key]['size_of_vocab'] = len(feats_max_min[key]["vocab"])
         if print_features and verbose > 1:
@@ -603,6 +616,7 @@ def classify_features_using_pandas(data_sample, target, model_options={}, verbos
     var_df1['int_cats'] = int_cats
     var_df1['int_bools'] = int_bools
     var_df1["continuous_vars"] = floats
+    var_df1['bools'] = bools
     #### It is better to have a baseline number for the size of the dataset here ########
     feats_max_min['DS_LEN'] = len(data_sample)
     return var_df1, feats_max_min
@@ -731,7 +745,7 @@ def find_latitude_columns(df, verbose=0):
                     continue
                 if df[sel_column].min() >= -90 and df[sel_column].max() <= 90:
                     if verbose:
-                        print('    %s found as latitude column' %sel_column)
+                        print('        %s found as latitude column' %sel_column)
                     if sel_column not in sel_columns:
                         sel_columns.append(sel_column)
                         lati_keyword = find_latitude_keyword(sel_column, columns, sel_columns)
@@ -744,7 +758,7 @@ def find_latitude_columns(df, verbose=0):
             for sel_column in sel_cols_string:
                 if len(df[df[sel_column].str.endswith(('N','S'))]) > 0:
                     if verbose:
-                        print('    %s found as latitude column' %sel_column)
+                        print('        %s found as latitude column' %sel_column)
                     if sel_column not in sel_columns:
                         sel_columns.append(sel_column)
                         lati_keyword = find_latitude_keyword(sel_column, columns, sel_columns)
@@ -877,7 +891,7 @@ def find_longitude_columns(df, verbose=0):
                     continue
                 if df[sel_column].min() >= -180 and df[sel_column].max() <= 180:
                     if verbose:
-                        print('    %s found as longitude column' %sel_column)
+                        print('        %s found as longitude column' %sel_column)
                     if sel_column not in sel_columns:
                         sel_columns.append(sel_column)
                         long_keyword = find_longitude_keyword(sel_column, columns, sel_columns)
@@ -890,7 +904,7 @@ def find_longitude_columns(df, verbose=0):
             for sel_column in sel_cols_string:
                 if len(df[df[sel_column].str.endswith(('N','S'))]) > 0:
                     if verbose:
-                        print('    %s found as longitude column' %sel_column)
+                        print('        %s found as longitude column' %sel_column)
                     if sel_column not in sel_columns:
                         sel_columns.append(sel_column)
                         long_keyword = find_longitude_keyword(sel_column, columns, sel_columns)
