@@ -257,7 +257,7 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
         ### load a small sample of data into a pandas dataframe ##
         train_small = pd.read_csv(train_datafile, sep=sep, nrows=maxrows, compression=compression,
                                 header=header, encoding=csv_encoding) 
-        ### this reads the entire file
+    ### this reads the entire file
     ##### Now detect modeltype if it is not given ###############
     try:
         modeltype = model_options["modeltype"]
@@ -288,6 +288,7 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
     
     label_encode_flag = False
     ##########  Find small details about the data to help create the right model ###
+
     if modeltype == 'Classification' or modeltype == 'Multi_Classification':
         if isinstance(target, str):
             #### This is for Single-Label problems ########
@@ -318,7 +319,12 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
     ### classify variables using the small dataframe ##
     print('    Classifying variables using data sample in pandas...')
     var_df1, cat_vocab_dict = classify_features_using_pandas(train_small, target, model_options, verbose=verbose)
-    
+
+    ##########    Just transfer all the values from var_df to cat_vocab_dict  ##################################
+    for each_key in var_df1:
+        cat_vocab_dict[each_key] = var_df1[each_key]
+    ############################################################################################################
+
     model_options['modeltype'] = modeltype
     model_options['model_label'] = model_label
     cat_vocab_dict['modeltype'] = modeltype
@@ -336,10 +342,6 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
                 keys=target_vocab, values=tf.constant(list(range(len(target_vocab))),
                                                dtype=tf.int64)),
             default_value=int(len(target_vocab)+1))
-
-        trans_output, cat_vocab_dict = transform_train_target(train_small, target, modeltype, 
-                                    model_label, cat_vocab_dict)
-        train_small[target] = trans_output.values
     
     #### Set column defaults while reading dataset from CSV files - that way, missing values avoided!
     ### The following are valid CSV dtypes for missing values: float32, float64, int32, int64, or string
@@ -354,6 +356,11 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
     float_cols = train_small.select_dtypes(include='float').columns.tolist()
     column_defaults = [-99.0 if x in float_cols else -99 if x in integer_cols else "missing" 
                                 for x in list(train_small)]
+    ####### Make sure you don't move this next stage. It should be after column defaults! ###
+    if label_encode_flag:
+        trans_output, cat_vocab_dict = transform_train_target(train_small, target, modeltype, 
+                                    model_label, cat_vocab_dict)
+        train_small[target] = trans_output.values
 
     #### CAUTION: (num_epochs=None) will automatically repeat the data forever! Be Careful with it!
     ### setting num_epochs to 1 is always good practice since it ensures that your dataset is readable later
@@ -456,7 +463,7 @@ def load_train_data_file(train_datafile, target, keras_options, model_options, v
     return train_small, data_batches, var_df1, cat_vocab_dict, keras_options, model_options
 ############################################################################################
 def to_ids(features, labels, table):
-    labels = tf.cast(labels, tf.int64)
+    #labels = tf.cast(labels, tf.int64) ## this should not have been used ##
     labels = table.lookup(labels)
     return (features, labels)
 #############################################################################################
@@ -464,6 +471,12 @@ def lenopenreadlines(filename):
     with open(filename) as f:
         return len(f.readlines())
 #########################################################################################
+def closest(lst, K):
+    """
+    Find a number in list lst that is closest to the value K.
+    """
+    return lst[min(range(len(lst)), key = lambda i: abs(lst[i]-K))]
+##########################################################################################
 def find_batch_size(DS_LEN):
     ### Since you cannot deal with a very large dataset in pandas, let's look into how big the file is
     maxrows = 10000
@@ -479,6 +492,8 @@ def find_batch_size(DS_LEN):
         batch_ratio = 0.0001
     batch_len = int(batch_ratio*DS_LEN)
     #print('    Batch size selected as %d' %batch_len)
+    lst = [32, 48, 64, 96, 128, 256]
+    batch_len = closest(lst, batch_len)
     return batch_len
 #########################################################################################
 def fill_missing_values_for_TF2(train_small, var_df):
@@ -498,14 +513,15 @@ def fill_missing_values_for_TF2(train_small, var_df):
     train_small = copy.deepcopy(train_small)
     cols_delete = var_df['cols_delete']
     cat_cols = var_df['categorical_vars'] + var_df['discrete_string_vars']
+    int_bools = var_df['int_bools']
     int_cats = var_df['int_cats']
     ints = var_df['int_vars']
-    int_cols = int_cats + ints
+    int_cols = int_cats + ints + int_bools
     float_cols = var_df['continuous_vars']
     nlp_cols = var_df['nlp_vars']
     date_vars = var_df['date_vars']
-    lats = var_df['lats']
-    lons = var_df['lons']
+    lats = var_df['lat_vars']
+    lons = var_df['lon_vars']
     ffill_cols = lats + lons + date_vars + cols_delete
 
     if len(cat_cols) > 0:
@@ -572,7 +588,7 @@ def load_train_data_frame(train_small, target, keras_options, model_options, ver
         target = ["_".join(x.split(" ")) for x in target ]
         model_label = 'Multi_Label'
     print('    Modified file names to fit no-space in column names rule in Tensorflow!')
-
+    #### if target is changed you must send that modified target back to other processes ######
     ### usecols is basically target in a list format. Very handy to know when target is a list.
         
     try:
@@ -592,13 +608,15 @@ def load_train_data_frame(train_small, target, keras_options, model_options, ver
     ###   Cat_Vocab_Dict contains all info about vocabulary in each variable and their size
     print('    Classifying variables using data sample in pandas...')
     var_df, cat_vocab_dict = classify_features_using_pandas(train_small, target, model_options, verbose=verbose)
-
+    ##########    Just transfer all the values from var_df to cat_vocab_dict  ##################################
+    for each_key in var_df:
+        cat_vocab_dict[each_key] = var_df[each_key]
+    ############################################################################################################
     model_options['modeltype'] = modeltype
     model_options['model_label'] = model_label
     cat_vocab_dict['target_variables'] = usecols
     cat_vocab_dict['modeltype'] = modeltype
     model_options['batch_size'] = batch_size
-
     ##########  Find small details about the data to help create the right model ###
     target_transformed = False
     if modeltype != 'Regression':
@@ -647,7 +665,7 @@ def load_train_data_frame(train_small, target, keras_options, model_options, ver
     cat_vocab_dict["target_transformed"] = target_transformed
 
     ####   fill missing values using this function ##############
-    train_small = fill_missing_values_for_TF2(train_small, var_df)
+    train_small = fill_missing_values_for_TF2(train_small, cat_vocab_dict)
     
     ##### Do the deletion of cols after filling with missing values since otherwise fill errors!
     drop_cols = var_df['cols_delete']
@@ -829,19 +847,29 @@ def load_train_data(train_data_or_file, target, project_name, keras_options, mod
     else:
         train_small, ds, var_df, cat_vocab_dict, keras_options, model_options = load_train_data_frame(train_data_or_file, target,
                                                                 keras_options, model_options, verbose)
+    ##### Just do a simple classification of the train_small data here ####################
+    #####  This might be useful for users to know whether to use feature-crosses or not ###
+    stri, numi = fast_classify_features(train_small)
+    #######################################################################################
+    #### if Target is modified in the above processes such as removing spaces, etc. you must re-init here
+    usecols = cat_vocab_dict['target_variables']
     cat_vocab_dict['DS_LEN'] = DS_LEN
     if verbose >= 1 and train_small.shape[1] <= 30:
         print_one_row_from_tf_dataset(ds)
     ####  Set Class Weights for Imbalanced Data Sets here ##########
     modeltype = model_options["modeltype"]
     #### You need to do this transform only for files. Otherwise, it is done already for dataframes.
-    if isinstance(target, str):
+    if len(usecols) == 1:
+        target = usecols[0]
+        ### This is a single label problem ########
         y_train = train_small[target]
         if modeltype != 'Regression' and not cat_vocab_dict['target_transformed']:
             cat_vocab_dict["original_classes"] = np.unique(train_small[target])
     else:
-        y_train = train_small[target[0]]
-        target_copy = copy.deepcopy(target)
+        ### This is a Multi-label label problem ########
+        target = usecols[0]
+        y_train = train_small[usecols[0]]
+        target_copy = copy.deepcopy(usecols)
         if modeltype != 'Regression' and not cat_vocab_dict['target_transformed']:
             for each_t in target_copy:
                 cat_vocab_dict[each_t+"_original_classes"] = np.unique(train_small[each_t])
@@ -910,3 +938,59 @@ def is_test(x, y):
 def is_train(x, y):
     return not is_test(x, y)
 ##################################################################################
+from collections import defaultdict
+import copy
+def fast_classify_features(df):
+    """
+    This is a very fast way to get a handle on what a dataset looks like. Just send in df and get a print.
+    Nothing is returned. You just get a printed number of how many types of features you have in dataframe.
+    """
+    num_list = df.select_dtypes(include='integer').columns.tolist()
+    float_list = df.select_dtypes(include='float').columns.tolist()
+    str_list = left_subtract(df.columns.tolist(), num_list+float_list)
+    all_list = [str_list, num_list]
+    str_dict = defaultdict(dict)
+    int_dict = defaultdict(dict)
+    for inum, dicti in enumerate([str_dict, int_dict]):
+        bincols = []
+        catcols = []
+        highcols = []
+        numcols = []
+        for col in all_list[inum]:
+            leng = len(df[col].value_counts())
+            if leng <= 2:
+                bincols.append(col)
+            elif leng > 2 and leng <= 15:
+                catcols.append(col)
+            elif leng >15 and leng <100:
+                highcols.append(col)
+            else:
+                numcols.append(col)
+        dicti['bincols'] = bincols
+        dicti['catcols'] = catcols
+        dicti['highcats'] = highcols
+        dicti['veryhighcats'] = numcols
+        if inum == 0:
+            str_dict = copy.deepcopy(dicti)
+            print('Distribution of string columns in datatset:')
+            print('    number of binary = %d, cats = %d, high cats = %d, very high cats = %d' %(
+                len(bincols), len(catcols), len(highcols), len(numcols)))
+        else:
+            print('Distribution of integer columns in datatset:')
+            int_dict = copy.deepcopy(dicti)
+            print('    number of binary = %d, cats = %d, high cats = %d, very high cats = %d' %(
+                len(bincols), len(catcols), len(highcols), len(numcols)))
+    ###   Check if worth doing cat_feature_cross_flag on this dataset ###
+    int_dict['floats'] = float_list
+    print('Distribution of floats: floats = %d' %len(float_list))
+    print('Data Transformation Advisory:')
+    if len(str_dict['bincols']+str_dict['catcols']) <= 10:
+        print('    perform categorical feature crosses')
+    if len(int_dict['bincols']+int_dict['catcols']) < 10:
+        print('    perform integer feature crosses')
+    if len(int_dict['veryhighcats']) > 0:
+        print('    transform %s from integer to float' %int_dict['veryhighcats'])
+    return str_dict, int_dict
+###################################################################################################
+
+###################################################################################################
