@@ -125,10 +125,12 @@ def fit(train_data_or_file, target, keras_model_type="basic", project_name="deep
     Inputs:
     train_data_or_file: can be file or pandas dataframe: you need to give path to filename.
     target: string or list. You can give one variable (string) or multiple variables (list)
-    keras_model_type: default = "auto". If you are not satisfied with the automatic model we build, 
-                        you can choose other options like 'Deep', 'Big Deep', 'Giant Deep', 
-                         'CNN1', 'CNN2', "BERT", etc. If you are using a BERT model, you must set
-                         the keras_model_type to "BERT". Otherwise, you will get an error.
+    keras_model_type: default = "fast". That will build a keras model and pipeline very fast.
+                    Then you can try other options like 'fast1', 'fast2' and finally 'auto'.
+                    You can also try 'CNN1', 'CNN2'. 
+                    If you are using it on NLP dataset, then set this to 'BERT' or 'USE'.
+                    'USE' stands for Universal Sentence Encoder. That's also a good model.
+                    Then it will automatically download a base BERT model and use it.
     project_name: default = "deep_autoviml". This is used to name the folder to save model.
     save_model_flag: default = False: it determines wher you want to save your trained model 
                     to local drive. If True, it will save it locally in project_name folder.
@@ -178,6 +180,8 @@ def fit(train_data_or_file, target, keras_model_type="basic", project_name="deep
             "cat_feat_cross_flag": if you want to cross categorical features such as A*B, B*C...
             "sep" : default = "," comma but you can override it. Separator used in read_csv.
             "idcols": default: empty list. Specify which variables you want to exclude from model.
+            "save_model_format": default is "" (empty string) which means tensorflow default .pb format: 
+                    Specify "h5" if you want to save it in ".h5" format.
             "modeltype": default = '': if you leave it blank we will automatically determine it.
                     If you want to override, your options are: 'Regression', 'Classification', 
                     'Multi_Classification'.
@@ -270,6 +274,7 @@ def fit(train_data_or_file, target, keras_model_type="basic", project_name="deep
     model_options_defaults = defaultdict(str)
     model_options_defaults["idcols"] = []
     model_options_defaults["modeltype"] = ''
+    model_options_defaults["save_model_format"] = "" 
     model_options_defaults["sep"] = ","
     model_options_defaults["cat_feat_cross_flag"] = False
     model_options_defaults["model_use_case"] = ''
@@ -298,7 +303,10 @@ def fit(train_data_or_file, target, keras_model_type="basic", project_name="deep
         print('Max Trials is 10 for faster processing. Please increase max_trials if you want more performance...')
         model_options["max_trials"] = 10
     else:
-        print('Max Trials : %s. Please lower max_trials if you want to run faster...' %model_options["max_trials"])
+        if model_options["max_trials"] <= 20:
+            print('Max Trials : %s. Please increase max_trials if you want to better accuracy...' %model_options["max_trials"])
+        else:
+            print('Max Trials : %s. Please lower max_trials if you want to run faster...' %model_options["max_trials"])
 
     print("""
 #################################################################################
@@ -346,7 +354,7 @@ def fit(train_data_or_file, target, keras_model_type="basic", project_name="deep
         ''')
     ######### this is where you get the model body either by yourself or sent as input ##
     fast_models = ['deep_and_wide','deep_wide','wide_deep', 'wide_and_deep','deep wide', 
-            'wide deep', 'fast', 'fast1', 'deep_and_cross', 'deep cross', 'deep and cross'] 
+            'wide deep', 'fast','fast1', 'fast2', 'deep_and_cross', 'deep cross', 'deep and cross'] 
     ##### This takes care of providing multi-output predictions! ######
     inputs = nlp_inputs+meta_inputs
     model_body, keras_options =  create_model(use_my_model, inputs, meta_outputs, 
