@@ -166,15 +166,20 @@ def print_one_row_from_tf_label(test_label):
             dict_row = list(test_label.as_numpy_iterator())[0]
         else:
             dict_row = test_label
-        print("Printing samples from labels data:")
         preds = list(dict_row.element_spec[0].keys())
-        labels = list(dict_row.element_spec[0].keys())
+        if dict_row.element_spec[1].name is None:
+            labels = ['target']
+        else:
+            labels = list(dict_row.element_spec[1].keys())
         if dict_row.element_spec[0][preds[0]].shape[0] is None or isinstance(
                 dict_row.element_spec[0][preds[0]].shape[0], int):
             for feats, labs in dict_row.take(1):
-                print(labs)
+                print("    samples from labels: %s" %(labs.numpy().tolist()[:10]))
     except:
-        print(list(test_label.as_numpy_iterator())[0])
+        print("    samples from each label in multi_label output: ")
+        for feats, labs in dict_row.take(1):
+            for each_label in labels:
+                print(labs[each_label])
 ###########################################################################################
 from sklearn.base import TransformerMixin
 from collections import defaultdict
@@ -430,7 +435,9 @@ def print_regression_model_stats(actuals, predicted, targets='', plot_name=''):
         mae, mae_asp, rmse_asp = print_regression_metrics(actuals, predicted)
     return mae, mae_asp, rmse_asp
 ################################################################################
+from sklearn.metrics import r2_score
 def print_regression_metrics(actuals, predicted):
+    predicted = np.nan_to_num(predicted)
     mae = mean_absolute_error(actuals, predicted)
     mae_asp = (mean_absolute_error(actuals, predicted)/actuals.std())*100
     rmse_asp = (np.sqrt(mean_squared_error(actuals,predicted))/actuals.std())*100
@@ -442,22 +449,24 @@ def print_regression_metrics(actuals, predicted):
     print('    RMSE = %0.4f' %rmse)
     print('    MAE as %% std dev of Actuals = %0.1f%%' %(mae/abs(actuals).std()*100))
     # Normalized RMSE print('RMSE = {:,.Of}'.format(rmse))
-    print('    Normalized RMSE (%% of MinMax of Actuals) = %0.0f%%' %(100*rmse/abs(actuals.max()-actuals.min())))
+    print('    R-Squared (%% ) = %0.0f%%' %(100*r2_score(actuals,predicted)))
     print('    Normalized RMSE (%% of Std Dev of Actuals) = %0.0f%%' %(100*rmse/actuals.std()))
     return mae, mae_asp, rmse_asp
 ################################################################################
-def print_static_rmse(actual, predicted, start_from=0,verbose=0):
+def print_static_rmse(actuals, predicted, start_from=0,verbose=0):
     """
     this calculates the ratio of the rmse error to the standard deviation of the actuals.
     This ratio should be below 1 for a model to be considered useful.
     The comparison starts from the row indicated in the "start_from" variable.
     """
-    rmse = np.sqrt(mean_squared_error(actual[start_from:],predicted[start_from:]))
-    std_dev = actual[start_from:].std()
+    predicted = np.nan_to_num(predicted)
+    rmse = np.sqrt(mean_squared_error(actuals[start_from:],predicted[start_from:]))
+    std_dev = actuals[start_from:].std()
     if verbose >= 1:
         print('    RMSE = %0.2f' %rmse)
         print('    Std Deviation of Actuals = %0.2f' %(std_dev))
         print('    Normalized RMSE = %0.1f%%' %(rmse*100/std_dev))
+        print('    R-Squared (%% ) = %0.0f%%' %(100*r2_score(actuals,predicted)))
     return rmse, rmse/std_dev
 ################################################################################
 from sklearn.metrics import mean_squared_error,mean_absolute_error
@@ -478,7 +487,6 @@ def print_mape(y, y_hat):
 from sklearn import metrics
 import matplotlib.pyplot as plt
 import copy
-import pdb
 def print_classification_header(num_classes, num_labels, target_name):
     ######## This is where you start printing metrics ###############
     if isinstance(num_classes, list) :
@@ -510,9 +518,9 @@ def print_classification_metrics(y_test, y_probs, proba_flag=True):
     else:
         y_preds = copy.deepcopy(y_probs)
     ##### check if it is multi-class #####
-    if len(np.unique(y_test)) > 2:
+    if len(np.unique(y_test)) > 2 or max(np.unique(y_test)) >= 2:
         multi_class_flag = True
-    
+    ###########  This is where we print the metrics ###################
     if not multi_class_flag  and not multi_label_flag:
         # Calculate comparison metrics for Binary classification results.
         accuracy = metrics.accuracy_score(y_test, y_preds)
@@ -822,7 +830,7 @@ def get_uncompiled_model(inputs, result, output_activation,
     #model_preprocessing = Model(inputs, meta_outputs)
     #preprocessed_inputs = model_preprocessing(inputs)
     #result = model_body(preprocessed_inputs)
-    ##### now you
+    ##### now you can add the final layer here #########
     multi_label_predictions = defaultdict(list)
     if num_labels <= 1:
         outputs = layers.Dense(num_predicts, activation=output_activation,
@@ -912,7 +920,7 @@ def get_callbacks(val_mode, val_monitor, patience, learning_rate, save_weights_o
     logdir = "deep_autoviml"
     callbacks_dict = {}
     tensorboard_logpath = os.path.join(logdir,"mylogs")
-    print('    Tensorboard log directory can be found at: %s' %tensorboard_logpath)
+    print('Tensorboard log directory can be found at: %s' %tensorboard_logpath)
 
     cp = keras.callbacks.ModelCheckpoint("deep_autoviml", save_best_only=True,
                                          save_weights_only=save_weights_only, save_format='tf')
@@ -935,23 +943,48 @@ def get_callbacks(val_mode, val_monitor, patience, learning_rate, save_weights_o
     es = keras.callbacks.EarlyStopping(monitor=val_monitor, min_delta=0.00001, patience=patience,
                         verbose=1, mode=val_mode, baseline=None, restore_best_weights=True)
     
-    tb = keras.callbacks.TensorBoard(tensorboard_logpath)
+    tb = keras.callbacks.TensorBoard(log_dir=tensorboard_logpath,
+                         histogram_freq=0,
+                         write_graph=True,
+                         write_images=True,
+                         update_freq='epoch',
+                         profile_batch=2,
+                         embeddings_freq=1
+                         )
 
     pr = PrintLR()
 
     callbacks_dict['onecycle'] = onecycle
     callbacks_dict['onecycle2'] = onecycle2
-    callbacks_dict['cp'] = cp
-    callbacks_dict['lr_sched'] = lr_sched
-    callbacks_dict['es'] = es
-    callbacks_dict['tb'] = tb
-    callbacks_dict['pr'] = pr
-    callbacks_dict['rlr'] = rlr
-    callbacks_dict['lr_decay_cb'] = lr_decay_cb
+    callbacks_dict['check_point'] = cp
+    callbacks_dict['scheduler'] = lr_sched
+    callbacks_dict['early_stop'] = es
+    callbacks_dict['tensor_board'] = tb
+    callbacks_dict['print'] = pr
+    callbacks_dict['reducer'] = rlr
+    callbacks_dict['decay'] = lr_decay_cb
 
     return callbacks_dict, tensorboard_logpath
 ####################################################################################
 def get_chosen_callback(callbacks_dict, keras_options):
+    ##### Setup Learning Rate decay.
+    #### Onecycle is another fast way to find the best learning in large datasets ######
+    ##########  This is where we look for various schedulers
+    if keras_options['lr_scheduler'] == "onecycle2":
+        lr_scheduler = callbacks_dict['onecycle2']
+    elif keras_options['lr_scheduler'] == 'onecycle':
+        lr_scheduler = callbacks_dict['onecycle']
+    elif keras_options['lr_scheduler'] == 'reducer':
+        lr_scheduler = callbacks_dict['reducer']
+    elif keras_options['lr_scheduler'] == 'decay':
+        lr_scheduler = callbacks_dict['decay']
+    elif keras_options['lr_scheduler'] == "scheduler":
+        lr_scheduler = callbacks_dict['scheduler']
+    else:
+        lr_scheduler = callbacks_dict['scheduler']
+    return lr_scheduler
+################################################################################################
+def get_chosen_callback2(callbacks_dict, keras_options):
     ##### Setup Learning Rate decay.
     #### Onecycle is another fast way to find the best learning in large datasets ######
     ##########  This is where we look for various schedulers
